package com.jeeplus.test.wdt.disposeRubbish.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.jeeplus.sys.domain.Office;
import com.jeeplus.sys.service.OfficeService;
import com.jeeplus.sys.service.dto.OfficeDTO;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.garbageClearance.domain.LookOver;
import com.jeeplus.test.garbageClearance.service.LookOverService;
import com.jeeplus.test.oss.domain.WorkAttachment;
import com.jeeplus.test.oss.domain.WorkAttachmentInfo;
import com.jeeplus.test.oss.service.OssService;
import com.jeeplus.test.wdt.disposeRubbish.domain.DisposeRubbish;
import com.jeeplus.test.wdt.disposeRubbish.domain.WorkOverNo;
import com.jeeplus.test.wdt.disposeRubbish.mapper.DisposeRubbishMapper;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.YearMonth;
import java.util.*;

/**
 * @author: 徐滕
 * @version: 2024-09-02 09:51
 */
@Service
@Transactional
public class DisposeRubbishService extends ServiceImpl<DisposeRubbishMapper, DisposeRubbish> {

    @Autowired
    private OssService ossService;
    @Autowired
    private LookOverService lookOverService;
    @Autowired
    private OfficeService officeService;

    public String getMaxNo() {
        // 创建查询条件构造器
        QueryWrapper<DisposeRubbish> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        queryWrapper.in("year",year);
        // 按no字段降序排列
        queryWrapper.orderByDesc("number");
        queryWrapper.last("limit 1");

        // 只获取第一条记录
        DisposeRubbish disposeRubbish = baseMapper.selectOne(queryWrapper);

        // 返回最大的no值
        return disposeRubbish != null ? disposeRubbish.getNo() : null;
    }

    public DisposeRubbish selectOne() {
        // 创建查询条件构造器
        QueryWrapper<DisposeRubbish> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        queryWrapper.in("year",year);
        // 按no字段降序排列
        queryWrapper.orderByDesc("number");
        queryWrapper.last("limit 1");

        // 只获取第一条记录
        DisposeRubbish disposeRubbish = baseMapper.selectOne(queryWrapper);

        // 返回最大的no值
        return disposeRubbish;
    }

    public UserDTO getUserInfoByOffId(String officeId) {
        return baseMapper.getUserInfoByOffId(officeId);
    }

    /**
     * 根据id获取巡查工单信息
     * @param id
     * @return
     */
    public DisposeRubbish selectById(String id) {
        DisposeRubbish disposeRubbish = baseMapper.selectById(id);
        //通过createById获取发起人所属村落。根据所属村落的运输距离和计费方式计算出补贴金额
        if(null != disposeRubbish && StringUtils.isNotBlank(disposeRubbish.getCreateById())){
            UserDTO userDTO = UserUtils.get(disposeRubbish.getCreateById());
            OfficeDTO officeDTO = userDTO.getOfficeDTO();
            if(null != officeDTO && StringUtils.isNotBlank(officeDTO.getId())){
                Office office = officeService.getById(officeDTO.getId());
                if(StringUtils.isNotBlank(office.getHaulDistance())){
                    disposeRubbish.setTransportMileage(office.getHaulDistance());
                    //字符串转数字
                    int haulDistance = Integer.parseInt(office.getHaulDistance());
                    if(haulDistance<=5){
                        //5公里内每公里补贴20元
                        disposeRubbish.setSubsidyUnitPrice(20);
                    } else if (haulDistance<=10){
                        //5-10公里每公里补贴22元
                        disposeRubbish.setSubsidyUnitPrice(22);
                    } else if (haulDistance<=20){
                        //10-20公里每公里补贴25元
                        disposeRubbish.setSubsidyUnitPrice(25);
                    }else{
                        //20公里以上每公里补贴28元
                        disposeRubbish.setSubsidyUnitPrice(28);
                    }
                }
            }
            System.out.println("");
        }
        if(null != disposeRubbish && StringUtils.isNotBlank(disposeRubbish.getProgramId())){
            // 使用 split() 方法将字符串分割为数组
            String[] array = disposeRubbish.getProgramId().split(",");
            // 将数组转换为 List
            List<String> programIdList = Arrays.asList(array);
            List<WorkAttachment> fileList1 = ossService.findFileList(programIdList, "before");
            disposeRubbish.setFileList1(fileList1);
            List<WorkAttachment> fileList2 = ossService.findFileList(programIdList, "between");
            disposeRubbish.setFileList2(fileList2);
            List<WorkAttachment> fileList3 = ossService.findFileList(programIdList, "after");
            disposeRubbish.setFileList3(fileList3);
            List<WorkAttachment> fileList4 = ossService.findFileList(id, "weigh");
            disposeRubbish.setFileList4(fileList4);

            List<WorkOverNo> workOverNoList = Lists.newArrayList();
            for (String workOverId : programIdList) {
                LookOver overServiceById = lookOverService.getById(workOverId);
                if(null != overServiceById && StringUtils.isNotBlank(overServiceById.getNo())){
                    WorkOverNo workOverNo = new WorkOverNo();
                    workOverNo.setId(overServiceById.getId());
                    workOverNo.setNo(overServiceById.getNo());
                    //workOverNo.setNo("XS-J" + overServiceById.getNo());
                    workOverNoList.add(workOverNo);
                }
            }
            disposeRubbish.setWorkOverNoList(workOverNoList);
        }
        // 根据id 从work_attachment 表中获取图片信息
        return disposeRubbish;
    }
    /**
     * 根据id获取巡查工单信息
     * @param procInsIdList
     * @return
     */
    public List<DisposeRubbish> getByProcInsId(List<String> procInsIdList) {
        return baseMapper.getByProcInsId(procInsIdList);
    }

    /**
     * 根据id获取巡查工单信息
     * @param programIdList
     * @return
     */
    public List<WorkAttachment> getByWorkOverId(List<String> programIdList) {
        // 根据id 从work_attachment 表中获取图片信息
        List<WorkAttachment> fileList1 = ossService.findFileList(programIdList, "before");
        return fileList1;
    }

    /**
     * 根据id获取巡查工单信息
     * @param officeId
     * @return
     */
    public List<DisposeRubbish> getDetailCollectByOfficeId(String officeId) {
        // 根据部门id查询所有年-月 信息汇总
        List<String> allYearMonthByOfficeIdList = baseMapper.getAllYearMonthByOfficeId(officeId);
        // 根据部门id查询所有工单信息
        List<DisposeRubbish> allYearMonthByOfficeIdDisposeRubbishList = baseMapper.getDisposeRubbishAllYearMonthByOfficeId(officeId);
        //根据发起年月对工单信息进行分组处理
        Map<String, List<DisposeRubbish>> temporaryListMap = dataMessageDispose(allYearMonthByOfficeIdDisposeRubbishList);
        //遍历该部门所有的年月信息，将年月信息和分组信息进行比对，并将分组信息进行汇总返回
        List<DisposeRubbish> dataList = Lists.newArrayList();
        for (String yearMonth : allYearMonthByOfficeIdList) {
            for (String key : temporaryListMap.keySet()) {
                if(StringUtils.isNotBlank(yearMonth) && yearMonth.equals(key)){
                    List<DisposeRubbish> mapList = temporaryListMap.get(key);
                    DisposeRubbish data = new DisposeRubbish();
                    data.setCreateDateStr(yearMonth);
                    data.setCount(mapList.size());
                    BigDecimal weightB = new BigDecimal(0);
                    BigDecimal subsidyB = new BigDecimal(0);
                    for (DisposeRubbish info : mapList) {
                        if(StringUtils.isNotBlank(info.getWeight())){
                            weightB = weightB.add(new BigDecimal(info.getWeight()));
                        }
                        if(StringUtils.isNotBlank(info.getSubsidy())){
                            subsidyB = subsidyB.add(new BigDecimal(info.getSubsidy()));
                        }
                    }
                    weightB = weightB.setScale(2, RoundingMode.HALF_UP);
                    subsidyB = subsidyB.setScale(2, RoundingMode.HALF_UP);
                    data.setWeight(weightB.toString());
                    data.setSubsidy(subsidyB.toString());

                    dataList.add(data);
                }
            }
        }

        if(dataList.size()>0){
            DisposeRubbish data = new DisposeRubbish();

            BigDecimal weightB = new BigDecimal(0);
            BigDecimal subsidyB = new BigDecimal(0);
            int count = 0;
            for (DisposeRubbish info : dataList) {
                count += info.getCount();
                if(StringUtils.isNotBlank(info.getWeight())){
                    weightB = weightB.add(new BigDecimal(info.getWeight()));
                }
                if(StringUtils.isNotBlank(info.getSubsidy())){
                    subsidyB = subsidyB.add(new BigDecimal(info.getSubsidy()));
                }
            }
            data.setCreateDateStr("汇总");
            data.setCount(count);
            weightB = weightB.setScale(2, RoundingMode.HALF_UP);
            subsidyB = subsidyB.setScale(2, RoundingMode.HALF_UP);
            data.setWeight(weightB.toString());
            data.setSubsidy(subsidyB.toString());
            dataList.add(data);
        }

        //遍历查询出来的数据，对数据进行汇总处理
        System.out.println();

        return dataList;
    }
    /**
     * 分组
     * @param list
     * @return
     */
    private Map<String, List<DisposeRubbish>> dataMessageDispose(List<DisposeRubbish> list) {
        Map<String, List<DisposeRubbish>> infoMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(list)) {
            // 分组
            for (DisposeRubbish vo : list) {
                List<DisposeRubbish> tempList = infoMap.get(vo.getAuditPassDateStr());
                /*如果取不到数据,那么直接new一个空的ArrayList**/
                if (tempList == null) {
                    tempList = new ArrayList<>();
                    tempList.add(vo);
                    infoMap.put(vo.getAuditPassDateStr(), tempList);
                }
                else {
                    /*某个vo之前已经存放过了,则直接追加数据到原来的List里**/
                    tempList.add(vo);
                }
            }
        }
        return infoMap;
    }


    /**
     * 根据id和年月获取巡查工单信息
     * @param officeId
     * @param yearMonth
     * @return
     */
    public List<DisposeRubbish> getDetailCollectByOfficeIdAndMonth(String officeId,String yearMonth) {
        // 将年月时间进行处理
        String startOfMonth = "";
        String endOfMonth = "";
        if(StringUtils.isNotBlank(yearMonth) && yearMonth.contains("-")){
            startOfMonth = yearMonth + "-01 00:00:00";
            // 使用 YearMonth 获取给定年月的最后一天
            YearMonth endyearMonth = YearMonth.parse(yearMonth);
            int lastDay = endyearMonth.lengthOfMonth(); // 当月的最后一天
            // 拼接上月份的最后一天并精确到时分秒
            endOfMonth = endyearMonth + "-" + lastDay + " 23:59:59";
        }
        //遍历该部门所有的年月信息，将年月信息和分组信息进行比对，并将分组信息进行汇总返回
        List<DisposeRubbish> dataList = Lists.newArrayList();
        if(StringUtils.isNotBlank(officeId) && StringUtils.isNotBlank(startOfMonth) && StringUtils.isNotBlank(endOfMonth)){

            // 根据部门id和起始时间查询所有工单信息
             dataList = baseMapper.getDetailCollectByOfficeIdAndMonth(officeId,startOfMonth,endOfMonth);
        }


        return dataList;
    }


    /**
     * 根据id获取巡查工单信息
     * @param no
     * @return
     */
    public DisposeRubbish getByNo(String no) {
        if (StringUtils.isNotBlank(no)){
            return baseMapper.getByNo(no);
        }
        return null;
    }

}
